#include "os.h"
#include "kheap.h"
#include "initrd.h"
#include "task.h"
#include "syscall.h"

extern page_directory_t* current_directory;
extern task_t* current_task;
extern tss_entry_t tss_entry;
extern ULONG read_eip();

int main()
{
    initial_esp = fetchESP();
    printformat("SS: %x, ESP: %x, EBP: %x, CS: %x, DS: %x\n", fetchSS(),fetchESP(),fetchEBP(),fetchCS(),fetchDS());
    k_clear_screen();
    printformat("Welcome to PrettyOS 0.07\n");
    printformat("initial_esp: %x \n", initial_esp);

    // GDT, IDT, ISRS, IRQ, timer, keyboard, paging, enable interrupts, RAM disk, multitasking

    settextcolor(2,0); printformat("GDT, IDT, ISRS, IRQ, timer, keyboard install\n"); settextcolor(15,0);
    gdt_install();
    idt_install();
    isrs_install();
    irq_install();
    initODA();
    timer_install();
    keyboard_install();
    sti();

    settextcolor(2,0); printformat("paging install\n"); settextcolor(15,0);
    paging_install();
    settextcolor(2,0); printformat("tasking install\n"); settextcolor(15,0);
    tasking_install();

    settextcolor(2,0); printformat("VFS & RAM Disk install\n"); settextcolor(15,0);
    ULONG ramdisk_start = 0x40081000, ramdisk_end = 0x40200000; // Im Heap Bereich
    if(placement_address < ramdisk_end) placement_address = ramdisk_end;
    printformat("placement_address after ram disk install: %x \n", placement_address);
    printformat("ramdisk_start: %x file_data_start: %x file_data_end: %x\n", ramdisk_start, &file_data_start, &file_data_end);
    k_memcpy((void*)ramdisk_start, &file_data_start, (ULONG)&file_data_end - (ULONG)&file_data_start); //process.asm

    fs_root = install_initrd(ramdisk_start);

    settextcolor(2,0); printformat("switch to user mode\n"); settextcolor(15,0);
    switch_to_user_mode();
    syscall_puts("Hello, user world!\n");

    // list the contents of /
    int i = 0;
    struct dirent* node = 0;
    while( (node = readdir_fs(fs_root, i)) != 0)
    {
        printformat("Found file %s\n",node->name);
        fs_node_t* fsnode = finddir_fs(fs_root, node->name);

        if((fsnode->flags & 0x7) == FS_DIRECTORY)
        {
            printformat("\t(directory)\n");
        }
        else
        {
            printformat("\t contents: \"");
            CHAR buf[256];
            ULONG sz = read_fs(fsnode, 0, fsnode->length, buf);
            int j;
            for(j=0; j<sz; ++j)
            printformat("%c",buf[j]);
            printformat("\"\n");
        }
        ++i;
    }

    settextcolor(15,0);
    for(i=0;i<3;++i)
    {
        fork(); //create a new process in a new address space which is a clone of this
    }
    sleepSeconds(10);

    //--------------------------------------------------

    k_clear_screen();
    settextcolor(getpid(),0);
    printformat("SS: %x, ESP: %x, EBP: %x, CS: %x, DS: %x\n", fetchSS(),fetchESP(),fetchEBP(),fetchCS(),fetchDS());

    pODA->ts_flag = 1;
    ULONG c=0;
    while(TRUE)
	{
	  if( k_checkKQ_and_print_char() )
	  {
	    ++c;
	    if(c>5)
	    {
	      c=0;
	      settextcolor(4,0);
	      printformat("\nT: %x H: %x WRITE: %i Read: %i ", pODA->pTailKQ, pODA->pHeadKQ, pODA->KQ_count_write, pODA->KQ_count_read);
          printformat("*T: %c %i *H: %c %i\n", *(pODA->pTailKQ),*(pODA->pTailKQ),*(pODA->pHeadKQ),*(pODA->pHeadKQ));
          settextcolor(2,0);
          settextcolor(getpid(),0);
	    }
	  }
	}
	return 0;
}
